#!/usr/bin/perl
use Data::Dumper;
use Getopt::Long;
use DBI;
use strict;

# Global variables

my $isHelp = 0;
my $database = undef;
my $rootdir = undef;

# General statistics variables
my $minTs = 9999999999999999999;
my $maxTs = 0;
my $numTcpPackets = 0;
my $numTcpSessions = 0;

# Statistics on dns 
my $dnsCntUUID = 0;
my $dnsCntStatic = 0;
my $dnsMinTs = 99999999999999999;
my $dnsMaxTs = 0;
my @flagList = ();

sub usage{
    my $exitcode = shift;
    print <<END
Do some general statistics from multiple databases generated from tcpick
log files

OPTIONS
    
    -h Shows this screen
    -d Specifies the sqlite3 database
    -r Specifies the root directory that contains multiple sqlite3 files
 
AUTHOR

    Gerard Wagener

LICENSE
 
    GPL
END
}

sub transform_timestamp{
    my $epoch = shift;
    my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) =gmtime($epoch);
    $year+=1900;
    $mon++;
    my $fmon=sprintf("%02d",$mon);
    my $fday=sprintf("%02d",$mday);
    my $fhour=sprintf("%02d",$hour);
    my $fmin=sprintf("%02d",$min);
    my $fsec=sprintf("%02d",$sec);
    return "$year-$fmon-$fday $fhour:$fmin:$fsec"; 
}
sub inspect_database{
    my $database = shift;
    my $dbargs = {AutoCommit => 1, PrintError => 1};
    my $dbh = DBI->connect("dbi:SQLite:dbname=$database","","",$dbargs) or 
        die "Cannot connect  to database\n";
    # Extract max,min and count at once to gain speed
    my $query="SELECT min(strftime(\"%s\",timestamp)), \ 
               max(strftime(\"%s\",timestamp)),count(*) FROM tcpick;";
    my $sth = $dbh->prepare($query) or die $dbh->errstr."\n";
    $sth->execute(); 
    my ($cmin, $cmax, $cnt);
    $sth->bind_columns(\$cmin,\$cmax,\$cnt);
    while ($sth->fetch()){
        if ($cmin < $minTs){
            $minTs = $cmin;
        }
        if ($cmax > $maxTs){
            $maxTs=$cmax;
        }
        $numTcpPackets+=$cnt;
    }
    # Extract established sessions
    my $query="SELECT count(*) FROM tcpick WHERE state=\"ESTABLISHED\";";
    my $sth = $dbh->prepare($query) or die $dbh->errstr."\n";
    $sth->execute(); 
    my $ses=0;
    $sth->bind_columns(\$ses);
    while ($sth->fetch()){
        $numTcpSessions+=$ses;
    }

    # Extract DNS queries
    my $query="SELECT COUNT(*) AS c FROM dns WHERE dnsqryname like 
              \"%exp.quuxlabs.com\" and not dnsqryname like 
              \"f.quuxlabs.com\";";
    my $sth = $dbh->prepare($query) or die $dbh->errstr."\n";
    $sth->execute() or die $dbh->errstr."\n";
    $sth->bind_columns(\$dnsCntUUID) or die $dbh->errstr."\n";
    $sth->fetch() or die $dbh->errstr."\n";
    my $query = "SELECT COUNT(*) AS c FROM dns WHERE dnsqryname like 
                 \"%f.exp.quuxlabs.com\";";

    my $sth = $dbh->prepare($query) or die $dbh->errstr."\n";
    $sth->execute() or die $dbh->errstr."\n";
    $sth->bind_columns(\$dnsCntStatic) or die $dbh->errstr."\n";
    $sth->fetch() or die $dbh->errstr."\n";

   my $query = "SELECT min(strftime(\"%s\",ts)),max(strftime(\"%s\",ts)) FROM dns WHERE dnsqryname like \"%exp.quuxlabs.com\";";
    my $sth = $dbh->prepare($query) or die $dbh->errstr."\n";
    $sth->execute() or die $dbh->errstr."\n";
    my ($mn,$mx);
    $sth->bind_columns(\$mn,\$mx) or die $dbh->errstr."\n";
    while($sth->fetch()){
        if ($mn < $dnsMinTs){
            $dnsMinTs = $mn;
        }
        if ($mx > $dnsMaxTs){
            $dnsMaxTs = $mx;
        }
    }
    my $query="SELECT DISTINCT dnsflags FROM dns WHERE dnsqryname 
               like \"%exp.quuxlabs.com\";\n";
    my $sth = $dbh->prepare($query) or die $dbh->errstr."\n";
    $sth->execute() or die $dbh->errstr."\n";
    my $flag;
    $sth->bind_columns(\$flag) or die $dbh->errstr."\n";
    
    while ($sth->fetch()){
        push(@flagList, $flag);
    } 
    $dbh->disconnect or die $dbh->errstr."\n";
}
GetOptions("help"=>\$isHelp, "root=s"=>\$rootdir, "database=s"=>\$database)
or exit 1;

if ($isHelp){
    &usage (0);
    exit 0;
}

# Is there only one database
if (defined($database)){
    &inspect_database($database);
}

if (defined($rootdir)){
    opendir H, "$rootdir" or die $?."\n";
    my @files  = readdir(H);
    foreach my $file (@files){
        if ($file=~/\.db$/){
            my $f = "$rootdir/$file";
            &inspect_database($f);
        }
    }
    close H;
}

my $loc_minTs = &transform_timestamp($minTs);
my $loc_maxTs = &transform_timestamp($maxTs);
# Dump results on stdout
print "*** General tcpick statistics ***\n";
print "Minimum timestamp (epoch): $minTs\n";
print "Minimum timestamp (gmtime): $loc_minTs\n";
print "Maximum timestamp (epoch): $maxTs\n";
print "Maximum timestamp (gmtime): $loc_maxTs\n";
print "Number of entries: $numTcpPackets\n";
print "Number of tcp sessions: $numTcpSessions\n";
print "*** General dns statistics ***\n";
print "Number of UUID dns queries: $dnsCntUUID\n";
print "Number of queries for the static image: $dnsCntStatic\n";
my $loc_dnsMinTs = &transform_timestamp($dnsMinTs);
my $loc_dnsMaxTs = &transform_timestamp($dnsMaxTs);
print "Minimum timestamp (epoch): $dnsMinTs\n";
print "Minimum timestamp (gmtime): $loc_dnsMinTs\n";
print "Maximum timestamp (epoch): $dnsMaxTs\n";
print "Maximum timestamp (gmtime): $loc_dnsMaxTs\n";
print "Used dns flags: ";
foreach my $f (@flagList){
    print sprintf('%x',$f)."  ";
}
print "\n";
exit 0;
