#!/usr/bin/perl
$|=101;
# 2009 Gerard Wagener
# License GPL
use Data::Dumper;
use Getopt::Long;
use DBI;
use strict;

my $create = undef;
my $isHelp = undef;
my $dbfile = undef;
my $root = undef;
my $dbh=undef;
my $shouldCreate = 0;
my $indexCreate = 0;

sub help{
    print <<END
Usage: tcp_log_match -d database -r root [-h]

Search for a filename based on information from tcpick log files

    -h --help     Shows this screen
    -d --database Specifies the database file
    -r --root     Specifies the root directory where are located the reassembled
                  files
    -c --create   Create table tcp content which describes the relation between
                  tcpick log entries with generated tcpick files
    -i --index    Create an index on the tcpick table

License
    GPL
Author
    Gerard Wagener
END
}
sub create_tables{
    my $dbargs = {AutoCommit => 0,
                  PrintError => 1};
    my $dbh = DBI->connect("dbi:SQLite:dbname=$dbfile","","",$dbargs);
    my $query="CREATE TABLE tcpcontent (id INTEGER PRIMARY KEY AUTOINCREMENT 
               NOT NULL, tid INTEGER NOT NULL, filename VARCHAR(200));";
    $dbh->do($query) or die $dbh->errstr."\n";
    # This table contains the ids of the log entries that could not be matched
    $query="CREATE TABLE tcpmismatch (id INTEGER PRIMARY KEY AUTOINCREMENT NOT
            NULL, tid INTEGER NOT NULL);";
    $dbh->do($query) or die $dbh->errstr."\n";
    $dbh->commit;
    $dbh->disconnect;
    print "Content tables were sucessfully created.\n";
    exit(0); 
}
# Tries to recover the generated tcpick filename.
# @returns the recovered filename on success
# @returns undef on error
sub getFileName{
    my ($sip, $sp, $dip, $dp, $logFile) = @_;
    my $rf = undef;
    # Are there multiple sessions per flow?
    my $query = "SELECT timestamp FROM tcpick WHERE 
                 sip=? and sp=? and dip=? AND dp=? AND filename=? AND
                 state=\"ESTABLISHED\" ORDER BY timestamp;";
    my $sth = $dbh->prepare($query) or  die $dbh->errstr."\n";
    $sth->execute($sip,$sp, $dip, $dp, $logFile) or die $dbh->errstr."\n"; 
    my $cnt = undef;
    $sth->bind_columns(\$cnt);
    my $c = 0;
    my $cf = undef;
    my $filename=$logFile;
    $filename=~s/\.log\.gz//g;
    while($sth->fetch()){
        if ($c > 0){
            $cf = sprintf("%x",$c);
            $rf = "$root/$filename/tcpick_$sip"."_".$dip."_";
            $rf.="$dp.serv.$cf.dat";
        }else{
            $rf = "$root/$filename/tcpick_$sip"."_".$dip."_"."$dp.serv.dat";
        }
        if (! -e $rf){
            return undef;
        }
        $c++;
    }
    # Format of tcpick files
    #tcpick_sip_dip_dp_idx.serv.idx.dat 
    #where idx is a linear counter of streams in hex
    return $rf;
}

sub store_content_match{
    my ($id,$filename) = @_;
    my $query="INSERT INTO tcpcontent (tid,filename) VALUES(?,?);";
    my $sth = $dbh->prepare($query) or die $dbh->errstr."\n";
    $sth->execute($id,$filename) or die $dbh->errstr."\n";
}

sub store_content_mismatch{
    my $id = shift;
    my $query = "INSERT INTO tcpmismatch (tid) VALUES (?);";
    my $sth = $dbh->prepare($query) or die $dbh->errstr."\n";
    $sth->execute($id) or die $dbh->errstr."\n";
}

# Parse command line arguments
GetOptions("help"=>\$isHelp,
           "dbfile=s"=>\$dbfile,
           "root=s"=>\$root,
           "create"=>\$shouldCreate,
           "index"=>\$indexCreate) or exit 1;

if (defined($isHelp)){
    &help;
    exit(0);
}
if ($shouldCreate){
    &create_tables;
    exit 0;

}
if (!defined($dbfile)){
    print STDERR "No database was specified\n";
    exit 1;
}
# Initialize database
my $dbargs = {AutoCommit => 0,
              PrintError => 1};
$dbh = DBI->connect("dbi:SQLite:dbname=$dbfile","","",$dbargs) or 
    die "Cannot connect  to database\n";


# Should an index be create to speed up queries?
if ($indexCreate){
    my $query="create index tcpickidx on tcpick (timestamp,state,sip,sp,dip,dp,filename);";
    my $sth = $dbh->prepare($query) or die $dbh->errstr."\n";
    $sth->execute() or die $dbh->errstr."\n";
    $dbh->commit() or die $dbh->errstr."\n";
    print "Successfully created index.\n";
}

# Go through established sessions
my $query="SELECT id,sip,sp,dip,dp,filename FROM tcpick WHERE 
           state=\"ESTABLISHED\"";
my $sth = $dbh->prepare($query) or die "Cannot prepare prepare query\n";
$sth->execute() or die $dbh->errstr;
my ($id,$sip,$sp,$dip,$dp,$filename)=(undef,undef,undef,undef,undef);
$sth->bind_columns(\$id, \$sip,\$sp,\$dip,\$dp,\$filename); 
while ($sth->fetch()){
    my $filename = &getFileName($sip,$sp,$dip,$dp,$filename);
    if (defined($filename)){
        # content was found
        &store_content_match($id,$filename);
    }else{
        &store_content_mismatch($id);
    }
}
$dbh->commit;
$dbh->disconnect;
