#!/usr/bin/perl
use strict;
use Data::Dumper;
use Getopt::Long;
use DBI;

# Global variables

# Global variables
my $isHelp = 0;
my $rootdir = undef;
my $database = undef;
my $shouldCreate = 0;
sub usage{
    my $exitcode = shift;
    print <<END;
Usage: inspect-payload [-h] [-r]  

Analyzes the payload of HTTP requests.

OPTIONS
    -h Shows this screen
    -r Specifies the root directory containing the tcpsessions
    -d Specifies the database that should be inspected
    -c Create http tables and exit

AUTHOR 
    Gerard Wagener

LICENSE
    GPL
END
    exit $exitcode;
}

sub inspect_payload{
    my $filename = shift;
    open G,"$filename" or die "$?\n";
    my $fields = {};
    while (<G>){
        chomp;
        my $line=$_;
        # Remove this \r
        $line=~s/\r//g;
        # Extract user-agent
        if ($line=~s/user-agent: //i){
            $fields->{'user-agent'} = $line;
        }
        if ($line=~s/^Host: //gi){
            $fields->{'host'}=$line;
        }
        # Extract cookies
        if ($line=~s/^Cookie: //gi){
            $fields->{'cookie'} = $line;
        }
        # Extract Accept-Language
        if ($line=~s/^Accept-Language: //gi){
            $fields->{'accept_language'} = $line;
        }
        # Extract Accept
        if ($line=~s/^Accept: //gi){
            $fields->{'accept'} = $line;
        }
        # Extract connection
        if ($line=~s/^Connection: //gi){
            $fields->{'connection'}=$line;
        }
        if ($line=~s/^Cache-Control: //gi){
            $fields->{'cache_control'} = $line;
        }
        if ($line=~s/^Pragma: //gi){
            $fields->{'pragma'}= $line;
        }
        # Handle HTTP requests
        if ($line=~s/HTTP\/(\d\.\d)$//g){
            $fields->{'version'} = $1; 
            if ($line ne ""){
                $line=~s/^([A-Z]*) //g;
                $fields->{'method'} = $1;
                $fields->{'uri'} = $line;
            } 
        }
    }
    close G;
    return $fields;
}

sub create_tables{
    my $dbargs = {AutoCommit => 0,
                  PrintError => 1};
    my $dbh = DBI->connect("dbi:SQLite:dbname=$database","","",$dbargs);
    my $query="CREATE TABLE httpreq (id INTEGER PRIMARY KEY AUTOINCREMENT 
               NOT NULL, filename VARCHAR(255), ua VARCHAR(200), 
               host VARCHAR(200), cookie TEXT, lang VARCHAR(200), 
               accept VARCHAR(200), con VARCHAR(30) ,CC VARCHAR(200), 
               pragma VARCHAR(64), meth VARCHAR(64), uri TEXT, 
               ver VARCHAR(32));";
    $dbh->do($query) or die $dbh->errstr."\n";
    # This table contains the ids of the log entries that could not be matched
    $dbh->commit;
    $dbh->disconnect; 
}


sub store_fields{
    my ($fields, $dbh) = @_;
    my $ua=$fields->{'user-agent'}."";
    my $host=$fields->{'host'}."";
    my $cookie=$fields->{'cookie'}."";
    my $accept_lang=$fields->{'accept_language'}."";
    my $accept = $fields->{'accept'}."";
    my $connection = $fields->{'connection'}."";
    my $cache_control = $fields->{'cache_control'}."";
    my $pragma = $fields->{'pragma'}."";
    my $version = $fields->{'version'}."";
    my $method = $fields->{'method'}."";
    my $uri = $fields->{ 'uri'}. "";
    my $filename = $fields->{'filename'};
    my $query="INSERT INTO httpreq (filename, ua, host, cookie, lang, accept,
               con, cc, pragma, meth, uri, ver) VALUES 
               (?,?,?,?,?,?,?,?,?,?,?,?)";
    my $sth = $dbh->prepare($query) or die $dbh->errstr."\n";
    $sth->execute($filename,$ua,$host,$cookie,$accept_lang, $accept, 
                    $connection, $cache_control, $pragma, $method, $uri, 
                    $version) or die $dbh->errstr."\n";
}

GetOptions("root=s"=>\$rootdir,"help"=>\$isHelp, 
           "database=s"=>\$database,"create"=>\$shouldCreate) or exit 1;

if ($isHelp){
    &usage(0);
}

# Check mandatory parameters
if (!defined($database)){
    print STDERR "A database must be specified\n";
    exit 1;
}
if ($shouldCreate){
    &create_tables;
    exit 0;
}

if (defined($rootdir)){
    my $dbargs = {AutoCommit => 0,
                  PrintError => 1};
    my $dbh = DBI->connect("dbi:SQLite:dbname=$database","","",$dbargs);

    open F,"ls $rootdir |" or die $?."\n";
    while (<F>){
        chomp;
        my $f = $_;
        my $c = "$rootdir/$f";
        my $fields = &inspect_payload($c);
        if (scalar(keys %{$fields}) > 0){
            $fields->{'filename'} = $c;
            &store_fields($fields,$dbh); 
       }
    }
    close F;
    $dbh->commit;
    $dbh->disconnect;
}


